"""

    PeopleList.py
    $Author: payne $
    $Date: 2002/04/18 18:49:42 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__verson__ = '$Revision: 1.11 $'[11:-2]

from LDAPExceptions import *
from STLDAPUtilities import _ldap, TRUE, FALSE, listCopy

class PeopleList:

    def __init__(self, user, queryString):
        
        self.user = user
        self.Connection = user.Connection
        self.searchBase = user.LDAPDirectory.getPersonSearchBase()
        managedSchema = user.LDAPDirectory.getPersonSchema()
        self.sortKey = ''
        self.sortKeyList = ['departmentNumber',
                            'displayName',
                            'mail',
                            'physicalDeliveryOfficeName',
                            'roomNumber',
                            'telephoneNumber',
                            'uid']

        for attributeName in listCopy(self.sortKeyList):
            attributeSchema = managedSchema.getAttributeSchema(attributeName)
            readers = attributeSchema.getReaders()
            canRead = FALSE
            for role in self.user.getRoles():
                if role in readers:
                    canRead = TRUE
                    break
            if not canRead:
                self.sortKeyList.remove(attributeName)
            

        filter = '(&(objectclass=person)'
        if queryString.find('ou=') == 0:
            filter = filter + '(ou=*' + queryString + '))'
        else:
            filter = filter + '(|'
            for item in self.sortKeyList:
                queryTerm = '(' + item + '=*' + queryString + '*)'
                filter = filter + queryTerm
            filter = filter + '))'
            
        try:
            results = self.Connection.search_s(self.searchBase,
                                               _ldap.SCOPE_SUBTREE,
                                               filter,
                                               self.sortKeyList)
        except _ldap.NO_SUCH_OBJECT:
            raise NO_SUCH_OBJECT

        self.resultsList = []
        for item in results:
            dn = item[0]
            attrs = item[1]
            attrs['distinguishedName'] = [dn]
            for key in self.sortKeyList:
                if not attrs.has_key(key):
                    attrs[key] = '-'
            self.resultsList.append(attrs)

    def sortBy(self, key):

        if key != self.sortKey:
            
            if key != 'distinguishedName':
                if not key in self.sortKeyList:
                    raise NO_SUCH_ATTRIBUTE
            
            self.sortKey = key
            self.resultsList.sort(self.__sortFunction)
        
        return self.resultsList

    def __sortFunction( self, x, y ):
        """
            Helper function to sort a list of dictionaries on an
            arbitrary dictionary key.  The dictionary items are
            assumed to be sequences and the sort is done on the
            first item in the sequence.
        """
        if x.has_key(self.sortKey):
            xval = x[self.sortKey][0]
        else:
            xval = ''

        if y.has_key(self.sortKey):
            yval = y[self.sortKey][0]
        else:
            yval = ''

        return cmp(xval.lower(), yval.lower())

    def value(self):

        outString = ''
        for attr in self.resultsList:
            
            dName = attr.get('displayName', [''])[0]
            outString = outString  + dName + ' ' * (25-len(dName))
            
            tels = attr.get('telephoneNumber', ['',''])
            outString = outString + tels[0] + ' ' * (10-len(tels[0]))
            
            if len(tels) > 1:
                altTel = tels[1]
            else:
                altTel = tels[0]                
            outString = outString + altTel + ' ' * (10-len(altTel))
            
            room = attr.get('roomNumber', [''])[0]            
            outString = outString + room + ' ' * (10-len(room))
            
            if attr.has_key('physicalDeliveryOfficeName'):
                mailStop = attr['physicalDeliveryOfficeName'][0]
            else:
                mailStop = attr.get('departmentNumber', [''])[0]
            outString = outString + mailStop +  ' ' * (10-len(mailStop))
            
            email = attr.get('uid', [''])[0].upper()
            outString = outString + email + ' ' * (10-len(email))
            
            branch = attr.get('departmentNumber', [''])[0]
            outString = outString + branch
            outString = outString +  '\n'
            
        return outString        
