"""

    STLDAPSyntax.py
    $Author: payne $
    $Date: 2002/04/18 15:28:35 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

       This module defines the valid LDAP attribute syntaxes and
       provides methods for converting between syntax OIDs and names.

"""

__version__ = '$Revision: 1.6 $'[11:-2]

_ldapSyntaxes = {
    '1.3.6.1.4.1.1466.115.121.1.1':       'ACIItem',
    '1.3.6.1.4.1.1466.115.121.1.2':       'AccessPoint',
    '1.3.6.1.4.1.1466.115.121.1.3':       'AttributeTypeDescription',
    '1.3.6.1.4.1.1466.115.121.1.4':       'Audio',
    '1.3.6.1.4.1.1466.115.121.1.5':       'Binary',
    '1.3.6.1.4.1.1466.115.121.1.6':       'Bitstring',
    '1.3.6.1.4.1.1466.115.121.1.7':       'Boolean',
    '1.3.6.1.4.1.1466.115.121.1.8':       'Certificate',
    '1.3.6.1.4.1.1466.115.121.1.9':       'CertificateList',
    '1.3.6.1.4.1.1466.115.121.1.10':      'CertificatePair',
    '1.3.6.1.4.1.1466.115.121.1.11':      'Country String',
    '1.3.6.1.4.1.1466.115.121.1.12':      'DN',
    '1.3.6.1.4.1.1466.115.121.1.13':      'DataQualitySyntax',
    '1.3.6.1.4.1.1466.115.121.1.14':      'DeliveryMethod',
    '1.3.6.1.4.1.1466.115.121.1.15':      'DirectoryString',
    '1.3.6.1.4.1.1466.115.121.1.16':      'DITContentRule',
    '1.3.6.1.4.1.1466.115.121.1.17':      'DITContentRuleDescription', 
    '1.3.6.1.4.1.1466.115.121.1.19':      'DSAQualitySyntax', 
    '1.3.6.1.4.1.1466.115.121.1.21':      'EnhancedGuide', 
    '1.3.6.1.4.1.1466.115.121.1.22':      'FacsimileTelephoneNumber',
    '1.3.6.1.4.1.1466.115.121.1.23':      'FacsimileImage',
    '1.3.6.1.4.1.1466.115.121.1.24':      'GeneralizedTime',
    '1.3.6.1.4.1.1466.115.121.1.25':      'Guide',
    '1.3.6.1.4.1.1466.115.121.1.26':      'IA5String',
    '1.3.6.1.4.1.1466.115.121.1.27':      'INTEGER',
    '1.3.6.1.4.1.1466.115.121.1.28':      'JPEGImage',
    '1.3.6.1.4.1.1466.115.121.1.30':      'MatchingRuleDescription',
    '1.3.6.1.4.1.1466.115.121.1.31':      'MatchingRuleUseDescription',
    '1.3.6.1.4.1.1466.115.121.1.34':      'NameAndOptionalUID',
    '1.3.6.1.4.1.1466.115.121.1.35':      'NameForm',
    '1.3.6.1.4.1.1466.115.121.1.36':      'NumericString',
    '1.3.6.1.4.1.1466.115.121.1.37':      'ObjectClassDescription',
    '1.3.6.1.4.1.1466.115.121.1.38':      'OID',
    '1.3.6.1.4.1.1466.115.121.1.39':      'OtherMailbox',
    '1.3.6.1.4.1.1466.115.121.1.40':      'OctetString',
    '1.3.6.1.4.1.1466.115.121.1.40{128}': 'Password',
    '1.3.6.1.4.1.1466.115.121.1.41':      'Postal Address',
    '1.3.6.1.4.1.1466.115.121.1.42':      'ProtocolInformation',
    '1.3.6.1.4.1.1466.115.121.1.43':      'Presentation Address',
    '1.3.6.1.4.1.1466.115.121.1.44':      'PrintableString',
    '1.3.6.1.4.1.1466.115.121.1.49':      'SupportedAlgorithm',
    '1.3.6.1.4.1.1466.115.121.1.50':      'TelephoneNumber',
    '1.3.6.1.4.1.1466.115.121.1.51':      'TeletexTerminalIdentifier',
    '1.3.6.1.4.1.1466.115.121.1.52':      'TelexNumber',
    '1.3.6.1.4.1.1466.115.121.1.53':      'UTCTime',
    '1.3.6.1.4.1.1466.115.121.1.54':      'LDAPSyntaxDescription',
    '1.3.6.1.4.1.4401.1.1.1':             'URI',
    }

def isValidOID(OID):
    if _ldapSyntaxes.has_key(OID):
        return 1
    elif OID.find('{') > -1 and _ldapSyntaxes.has_key( OID[:OID.find('{')] ):
        return 1
    return 0

def getNameFromOID(OID):
    """ Returns the syntax name for a given OID. """
    
    if _ldapSyntaxes.has_key(OID):
        return _ldapSyntaxes[OID]        
    elif OID.index('{') > 0 and _ldapSyntaxes.has_key( OID[:OID.index('{')] ):
        return _ldapSyntaxes[OID[:OID.index('{')]]        

def getOIDFromName(Name):
    """ Returns the OID for the named syntax. """

    tmp = [OID for OID in _ldapSyntaxes.keys()
           if _ldapSyntaxes[OID].lower() == Name.lower()]

    assert len(tmp) == 1
    return tmp[0]

def getCanonicalName(Name):
    """ Returns the canonical name of the syntax with
        propert attention to case. """
    
    return getNameFromOID(getOIDFromName(Name))

    
