"""

    Module: $RCSfile: ZRoles.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/02/14 02:00:03 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""

class ZRoles:

    def __init__(self, user, managedRoles ):

        self._user = user
        self._managedRoles = managedRoles

    def updateRolesFromRequest( self, REQUEST ):
        """
            Change the roles according to the REQUEST variables.
        """
        message = None

        #   Invoke a method that matches the name of the submit
        #   button (with '_' prepended).

        if REQUEST.has_key( 'addRoles' ):
            message = self._addRoles( REQUEST )

        elif REQUEST.has_key( 'deleteConfiguredRoles' ):
            message = self._deleteRoles( REQUEST )

        elif REQUEST.has_key( 'renamePermanentRoles' ):
            message = self._renameRoles( REQUEST )

        else:
            # You can get here if your form has a single text input
            # field and you press return rather than the form button.
            # Maybe there is a way to figure it out:

            message = "No method to perform requested action."

        return message

    def _addRoles( self, REQUEST ):

        # Add configured roles, given a list of names.

        roleNames = REQUEST.get( 'newRoles' )

        for role in roleNames:
            self._managedRoles.addRole( role )

    def _deleteRoles( self, REQUEST ):

        # Delete configured roles, given the names

        roleNames = REQUEST.get( 'ids' )

        # Easy: delete from the list

        for role in roleNames:
            self._managedRoles.delRole( role )

        # Hard: look through all of the attribute reader and writer
        # lists in all of the schema and delete from them.

        ldapDirectory = self._user.getLDAPDirectory()
        for schema in (ldapDirectory.getPersonSchema(), 
                       ldapDirectory.getOrgUnitSchema(),
                       ldapDirectory.getGroupSchema() ):

            for ldapName in schema.getAllowedAttributeNames():
                schema.getAttributeSchema(ldapName).delReaders( roleNames )
                schema.getAttributeSchema(ldapName).delWriters( roleNames )

        message = "Deleted role(s)."

    def _changeRoles( self, REQUEST ):

        # A method for updating the roles list.

        option = REQUEST.get( 'changeRoles' )
        ids = REQUEST.get( 'ids' )

        if option == " Delete " and len( ids ) > 0:
            for item in ids:
                if item in self._managedRoles.getConfiguredRoleNames():
                    self._managedRoles.delRole( item )
            message = "Deleted configured roles."

        elif option == " Add ":
            newRole = REQUEST.get( 'newRole', None )
            if newRole:
                self._managedRoles.addRole( newRole )
            message = "Added role."

        else:
            message = ""

        return message

    def _renameRoles( self, REQUEST ):
        """
            Rename roles.
        """
        #   If we got here from the _schemaRenameRolesForm, then 
        #   perform the rename.

        if REQUEST.has_key('haveNewNames'):
            canonicalNames = REQUEST.get( 'ids', [] )
            newNames = REQUEST.get( 'new_ids', [] )

            # Skip empty new_ids

            while '' in newNames:
                i = newNames.index('')
                canonicalNames.pop(i)
                newNames.pop(i)

            for i in range( 0, len( canonicalNames ) ):
                canonicalName = canonicalNames[ i ]
                newName = newNames[ i ]

                self._managedRoles._permanentRoles[canonicalName]['roleName'] =\
                    newName

            return "Renamed roles categories.  " 

        #   Otherwise, set REQUEST variables so that the
        #   _schemaRenameCategoriesForm gets displayed.

        else:
            REQUEST.set('schemaFormID', 'schemaRenameRoles')
            del REQUEST.form['updateSchema']

            return ""


