"""

    ZUtilities.py
    $Author: stevray $
    $Date: 2002/02/13 18:24:07 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

    A repository for Zope level (Zope-aware?) utility methods.

"""

__version__ = '$Revision: 1.8 $'[11:-2]

from AccessControl import getSecurityManager
from Connection import AnonymousUser, AuthenticatedUser
from DocumentTemplate.DocumentTemplate import HTML
from LDAPExceptions import *
from urllib import urlencode

class ZUtilities:

    def _getUser(self, ldapDirectory, REQUEST):
        """
            Return an AuthenticatedUser connection object in the case of
            an LDAP authenticated user.  But because authenticated users can 
            change their own passwords, the Zope user object can be holding 
            on to invalid credentials.  If that is the case, then expire 
            the Zope user, and return an AnonymousUser connection.  Users
            not already logged in also get an AnonymousUser connection.
        """

        if REQUEST.has_key( 'LDAPUser' ):
            return REQUEST['LDAPUser']

        user = getSecurityManager().getUser()

        if hasattr( user, '_getPassword' ):
            uid = user.getUserName()
            pw = user._getPassword() 
            try:
                LDAPUser = AuthenticatedUser( uid, pw, ldapDirectory)
            except INVALID_CREDENTIALS:
                if hasattr( user, '_expire' ):
                    user._expire()
                raise 'Unauthorized'
            except:
                LDAPUser = AnonymousUser( ldapDirectory )
        else:            
            LDAPUser = AnonymousUser( ldapDirectory )
        REQUEST.set( 'LDAPUser', LDAPUser )
        return LDAPUser

    def buildTree(self, user, REQUEST, startingDN=None,
                  sort="",
                  branchMethod="getChildGroups",
                  action="?",
                  returnsToFormID='display' ):
        """
            Generate a tree view of groups in the directory.
        """
        # The trick is to use the urlparam of the tree tag to
        # hold all of the request variables except for the tree
        # state.  But the tree tag parameters are stupidly parsed
        # as a string, so you can't pass it a DTML variable
        # containing this information.  So I end up generating
        # a string that contains the DTML with the right parameters,
        # create a temporary DocumentTemplate.HTML object, and
        # render it in the context of this method.

        # queryString holds just the form variables in the REQUEST.
        #
        if REQUEST.has_key('queryString'):
            del REQUEST.form['queryString']
        if REQUEST.has_key('tree-c'):
            del REQUEST.form['tree-c']
        if REQUEST.has_key('tree-e'):
            del REQUEST.form['tree-e']

        targetAttribute = REQUEST['targetAttribute']
        if REQUEST.form.has_key(targetAttribute):
            del REQUEST.form[targetAttribute]

        queryString = urlencode( REQUEST.form, 1 )

        if startingDN is None:
            startingDN = self.LDAPDirectory.getDirectoryBase()

        # Finally, the REQUEST has a key that brings us back
        # to the tree.  We need to remove that key for the version
        # of the query string used in the tree's links.

        if REQUEST.has_key( 'selectDN_'+targetAttribute ):
            del REQUEST.form['selectDN_'+targetAttribute]
        REQUEST.set('objectFormID', returnsToFormID )

        queryStringReturn = urlencode( REQUEST.form, 1 )

        REQUEST.set( 'user', user )

        # Here's a string template that holds the DTML returned by
        # this method.  Notice the replacement strings at the end.
        #
        treeString = """
          <dtml-with user>
          <dtml-call "REQUEST.set('mappingDN', '%s')">
          <dtml-tree urlparam="%s"
                     id="mappingDN"
                     sort="%s"
                     branches_expr="%s(mappingDN)">
            <a href="&dtml-URL0;%s%s&%s=<dtml-var "mappingDN" url_quote>">&dtml-%s;</a>
          </dtml-tree>
          </dtml-with> """ % \
                           ( startingDN,
                             queryString,
                             sort,
                             branchMethod,
                             action,
                             queryStringReturn,
                             REQUEST['targetAttribute'],
                             sort
                           )

        #return treeString

        # Make a DocumentTemplate.HTML object out of the string, and
        # render it.
        #
        ob = HTML( treeString )
        return apply(HTML.__call__, ( ob, self, REQUEST ))
