"""
    formDisplayManager.py
    $Author: payne $
    $Date: 2002/02/20 13:56:46 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__verion__ = 'Revision: $'[11:-2]

from AccessControl import ClassSecurityInfo, getSecurityManager
from Globals import DTMLFile, InitializeClass
from urllib import quote_plus, unquote_plus

from STLDAPUtilities import TRUE, FALSE
from ZObject import ZObject

class formDisplayManager:

    manage_options = ( { 'label'  : 'Search',
                         'action' : '',
                         'help'   : ('STLDAPManager', 'Search.stx' ) }, )

    security = ClassSecurityInfo()

    security.declarePublic('index_html')

    _browseForm = DTMLFile('dtml/displayBrowse', globals())
    _displayForm = DTMLFile('dtml/displayObject', globals())
    _searchForm = DTMLFile('dtml/displaySearch', globals())
    _selectPersonDNForm = DTMLFile('dtml/selectPersonDN', globals())
    _selectOrgUnitDNForm = DTMLFile('dtml/selectOrgUnitDN', globals())
    _selectGroupDNForm = DTMLFile('dtml/selectGroupDN', globals())

    def index_html( self, REQUEST, message=None ):
        """
            Given a REQUEST containing a distinguished name, display it.
            Otherwise show the search form.
        """

        if REQUEST.has_key( 'distinguishedName' ):
            distinguishedName = unquote_plus( REQUEST.get('distinguishedName', '' ))
            if not REQUEST.has_key('objectFormID'):
                REQUEST.set('objectFormID', 'display' )
        elif not REQUEST.has_key( 'objectFormID' ):
            REQUEST.set('objectFormID', 'search' )

        #   Create a ZObject, which wraps an LDAPObject.

        user = self._getUser( self.LDAPDirectory, REQUEST )
        zObject = ZObject( user, REQUEST )

        #   If we got here by a form submit button, then we need to
        #   update the directory first.

        if REQUEST.has_key('updateObject'):
            if not REQUEST.has_key('cancel'):
                message = zObject.updateDirectoryFromRequest( REQUEST )

                if zObject.iChangedMyOwnPassword():
                    #   ... then I can't go on.  Expire the Zope user 
                    #   object, and redirect to the search screen,  
                    #    where I can login again.

                    getSecurityManager().getUser()._expire()
                    zObject._login( REQUEST )

            else:
                #   Clean up the REQUEST object by just doing a
                #   redirect back to this form for this object.

                url = "%s?distinguishedName=%s" % \
                      ( REQUEST.URL1,
                        quote_plus(distinguishedName) )
                if REQUEST.has_key( 'displayCategory' ):
                    url = "%s&displayCategory=%s" % ( url,
                                      quote_plus(REQUEST.get('displayCategory')) )
                REQUEST.RESPONSE.redirect( url )

        #   Set default form, and render it.  Each form must contain a
        #   hidden tag called 'schemaFormID' whose value, with '_'
        #   prepended, and 'Form' appended, is form rendering DTMLFile
        #   call.

        #   An anonymous user should not be able to view the display
        #   form.

        if REQUEST.objectFormID == 'display' and user.distinguishedName != "":
            return self._displayForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'browse':
            return self._browseForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'selectPersonDN':
            return self._selectPersonDNForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'selectOrgUnitDN':
            return self._selectOrgUnitDNForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'selectGroupDN':
            return self._selectGroupDNForm(
                        self, REQUEST, manage_tabs_message = message )

        else:
            return self._searchForm( 
                        self, REQUEST, )
 
    def logout( self, REQUEST ):
        """
            Logout.
        """
        REQUEST.RESPONSE.setStatus(401)
        REQUEST.RESPONSE.setHeader('WWW-Authenticate','basic realm=')
        return 'Logged out.'

InitializeClass(formDisplayManager)    
