"""
    formObjectManager.py
    $Author: payne $
    $Date: 2002/02/20 13:56:46 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__verion__ = 'Revision: $'[11:-2]

from AccessControl import ClassSecurityInfo
from Globals import DTMLFile, InitializeClass
from urllib import quote_plus

from STLDAPUtilities import TRUE, FALSE
from ZObject import ZObject

class formObjectManager:

    manage_options = ( { 'label'  : 'People',
                         'action' : 'manage_people',
                         'help'   : ('STLDAPManager', 'Object.stx' ) }, 
                       { 'label'  : 'Org Units',
                         'action' : 'manage_orgUnits',
                         'help'   : ('STLDAPManager', 'Object.stx' ) }, 
                       { 'label'  : 'Groups',
                         'action' : 'manage_groups',
                         'help'   : ('STLDAPManager', 'Object.stx' ) }, )

    security = ClassSecurityInfo()

    security.declareProtected( 'LDAP Manage People', 'manage_people' )
    security.declareProtected( 'LDAP Manage Org Units', 'manage_orgUnits')
    security.declareProtected( 'LDAP Manage Groups', 'manage_groups' )
    security.declarePrivate( 'manage_objects' )
    security.declarePrivate( 'manage_objectFormSelection' )

    #   DTML templates for the various "faces" of the Objects tab.
    #   Each template's forms must contain a hidden variable that
    #   identifies the form.  The name is 'objectFormID', and the
    #   value must match one of the tests for it in _manage_objects.
    #
    #     <input type=hidden name=objectFormID value=objectAdd>
 
    manage_objectFormSelection = DTMLFile('dtml/objectFormSelection', globals())
    _browseGroupsForm = DTMLFile('dtml/browseGroups', globals())
    _browseOrgUnitsForm = DTMLFile('dtml/browseOrgUnits', globals())
    _objectAddForm = DTMLFile('dtml/objectAdd', globals())
    _objectDelForm = DTMLFile('dtml/objectDel', globals())
    _objectAddMembersForm = DTMLFile('dtml/objectAddMembers', globals())
    _objectRemoveMembersForm = DTMLFile('dtml/objectRemoveMembers', globals())
    _personDelForm = DTMLFile('dtml/objectDelPerson', globals())
    _selectPersonOrgUnitDNForm = DTMLFile('dtml/selectPersonOrgUnitDN',globals())

    def manage_people( self, REQUEST, message=None ):
        """
            Throw up the people management form.
        """
        REQUEST.set('objectCategory',     'person' )
        schema = self.LDAPDirectory.getPersonSchema()

        return self._manage_objects( schema, REQUEST, message )

    def manage_orgUnits( self, REQUEST, message=None ):
        """
            Throw up the orgUnit management form.
        """
        REQUEST.set('objectCategory',     'orgUnit' )
        schema = self.LDAPDirectory.getOrgUnitSchema()

        return self._manage_objects( schema, REQUEST, message )

    def manage_groups( self, REQUEST, message=None ):
        """
            Throw up the group management form.
        """
        REQUEST.set('objectCategory',     'group' )
        schema = self.LDAPDirectory.getGroupSchema()

        return self._manage_objects( schema, REQUEST, message )

    def _manage_objects( self, schema, REQUEST, message=None ):

        #   Put into the REQUEST information needed on the forms
       
        namingAttribute = schema.getNamingAttribute()

        REQUEST.set('namingAttribute',    namingAttribute )
        REQUEST.set('friendlyNamingAttribute',
              schema.getAttributeSchema(namingAttribute).getFriendlyName() )
        REQUEST.set('objectClasses',
              schema.getAttributeSchema('objectClass').getRequiredValues() )
        REQUEST.set('requiredAttributes', schema.getRequiredAttributeNames() )
        REQUEST.set('controllers',        schema.getControllers() )
    
        if not REQUEST.has_key('objectFormID'):
            REQUEST.set('objectFormID','addObject')

        #   Instantiate a ZObject for the form.

        user = self._getUser( self.LDAPDirectory, REQUEST )
        zObject = ZObject( user, REQUEST )

        #   If a distinguishedName was specified, refresh the object
        #   with information from the directory at that DN.

        if REQUEST.has_key('distinguishedName'):
            distinguishedName = REQUEST.get('distinguishedName')
            zObject._object.refresh( distinguishedName )

        #   If we got here by a form submit button, then we need to
        #   update the directory first.  Any form that needs to hit
        #   the methods in ZObject.updateDirectoryFromRequest must
        #   contain:
        #
        #   <input type="hidden" name="updateObject" value="1">

        if REQUEST.has_key('updateObject'):
            if not REQUEST.has_key('cancel'):
                message = zObject.updateDirectoryFromRequest( REQUEST )
                zObject = ZObject( user, REQUEST )

            else:
                #   Clean up the REQUEST object by just doing a
                #   redirect back to this form for this object.

                url = "%s?objectFormID=%s" % \
                      ( REQUEST['URL0'],
                        REQUEST['objectFormID'] )
                if REQUEST.has_key('distinguishedName'):
                    url = url + "&distinguishedName=%s" % \
                      ( quote_plus(REQUEST['distinguishedName']) )
                REQUEST.RESPONSE.redirect( url )

        #   Set default form, and render it.  Each form must contain a
        #   hidden tag called 'schemaFormID' whose value, with '_'
        #   prepended, and 'Form' appended, is form rendering DTMLFile
        #   call.

        if REQUEST.objectFormID == 'addObject':
            return self._objectAddForm( 
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'delObject':
            if REQUEST['objectCategory'] == 'person':
                return self._personDelForm(
                        self, REQUEST, manage_tabs_message = message )
            else:
                return self._objectDelForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'addMembers':
            return self._objectAddMembersForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'removeMembers':
            return self._objectRemoveMembersForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'selectPersonDN':
            return self._selectPersonDNForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'selectPersonOrgUnitDN':
            return self._selectPersonOrgUnitDNForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'selectOrgUnitDN':
            return self._selectOrgUnitDNForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'selectGroupDN':
            return self._selectGroupDNForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'browseOrgUnits':
            return self._browseOrgUnitsForm(
                        self, REQUEST, manage_tabs_message = message )

        elif REQUEST.objectFormID == 'browseGroups':
            return self._browseGroupsForm(
                        self, REQUEST, manage_tabs_message = message )

        else:
            message = """
                      The previous form's objectFormID does not match a
                      known form display method.
                      """
            return self._objectAddForm(
                        self, REQUEST, manage_tabs_message = message )
 
InitializeClass(formObjectManager)    
