"""

    Module: $RCSfile: TestAnonymousUser.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/04/18 14:54:38 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.13 $'[11:-2]

import sys
sys.path.insert(0,'..')

from TestDirectory import getTestDirectory
from Connection import *
import unittest

class testAnonymousUser(unittest.TestCase):

    """ Tests the AnonymousUser class in the Connection module. """

    def setUp(self):
        self.nobody = AnonymousUser(getTestDirectory())

    def test01(self):
        """ 01 testAnonymousUser: Anonymous bind test """
        self.failUnless(self.nobody)

    def test02(self):
        """ 02 testAnonymousUser: getDNfromUserName for valid user """
        dn = self.nobody.getDNfromUserName('usera')
        self.failUnless(dn.lower() == 'uid=usera,ou=people,o=test'.lower())

    def test03(self):
        """ 03 testAnonymousUser: getDNfromUserName for invalid user """
        self.assertRaises(NO_SUCH_OBJECT, self.nobody.getDNfromUserName, 'nonexistentuser')

    def test04(self):
        """ 04 testAnonymousUser: unbind test """
        self.nobody.unbind()
        self.assertRaises(NO_SUCH_OBJECT, self.nobody.getDNfromUserName, 'uid=usera,ou=people,o=test')

    def test05(self):
        """ 05 testAnonymousUser: test getChildGroups """
        testGroups = [MappingObject('cn=Directory Administrators,ou=Groups,o=test'),
                      MappingObject('cn=Divisional Administrators,ou=Groups,o=test'),
                      MappingObject('cn=Empty Group,ou=Groups,o=test'),
                      MappingObject('cn=Manager,ou=Groups,o=test'),
                      MappingObject('cn=System Administrators,ou=Groups,o=test'),
                      MappingObject('cn=Test Group,ou=Groups,o=test')]
        groups = self.nobody.getChildGroups("ou=Groups,o=test")
        self.failUnless( len(groups) == len(testGroups) )
        indexes = range(len(groups))
        for i in indexes:
            t1 = testGroups[i]
            t2 = groups[i]
            self.failUnless(t1.mappingName.lower() == t2.mappingName.lower())
            self.failUnless(t1.mappingDN.lower() ==\
                            t2.mappingDN.lower() )

    def test06(self):
        """ 06 testAnonymousUser: test getChildOrgUnits """
        testOUs = [MappingObject( 'ou=Groups,o=test' ),
                   MappingObject( 'ou=People,o=test' ),
                   MappingObject( 'ou=Permanent Test OU,o=test' ),
                   MappingObject( 'ou=ST ScI,o=test' ) ]
        orgUnits = self.nobody.getChildOrgUnits("o=test")
        self.failUnless( len(orgUnits) == len(testOUs) )
        indexes = range(len(orgUnits))
        for i in indexes:
            t1 = testOUs[i]
            t2 = orgUnits[i]
            self.failUnless(t1.mappingName.lower() == t2.mappingName.lower())
            self.failUnless(t1.mappingDN.lower() ==\
                            t2.mappingDN.lower() )
        
if __name__ == '__main__':
    
    tests = unittest.makeSuite(testAnonymousUser, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
