"""

    Module: $RCSfile: TestAttributeSchema.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/04/18 14:54:38 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.14 $'[11:-2]

import sys
sys.path.insert(0,'..')

import unittest
from ManagedAttributeSchema import ManagedAttributeSchema, ManagedAttribute
from SchemaDefaults import defaultUserSchema
from STLDAPUtilities import *
import STLDAPSyntax

class testAttributeSchema(unittest.TestCase):
    """ Tests the ManagedAttributeSchema and ManagedAttribute classes. """

    def setUp(self):
        self.attrSchema = ManagedAttributeSchema('cn',defaultUserSchema['cn'])

    def test01(self):
        """ 01 AttributeSchema: testing getAttributeName """
        self.failUnless( self.attrSchema.getAttributeName() == 'cn')

    def test02(self):
        """ 02 AttributeSchema: testing getAvailableValues """
        values = ['me', 'you']
        self.failUnless( self.attrSchema.getValidValues() == [] )
        self.attrSchema.setValidValues(values)
        self.failUnless( self.attrSchema.getRequiredValues() == [] )
        self.attrSchema.setRequiredValues(['me',])
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( 
                  listsAreEquivalent( self.attrSchema.getAvailableValues(), 
                                      ['you',] ) )

    def test03(self):
        """ 03 AttributeSchema: testing get/set FriendlyName """
        self.failUnless( self.attrSchema.getFriendlyName() == 'Common Name(s)')
        self.attrSchema.setFriendlyName('Common Name')
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( self.attrSchema.getFriendlyName() == 'Common Name')

    def test04(self):
        """ 04 AttributeSchema: testing get/set Managed """
        self.attrSchema.setManaged( TRUE )
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( self.attrSchema.getManaged() )
        self.attrSchema.setManaged( FALSE )
        self.failIf( self.attrSchema.getManaged() )

    def test05(self):
        """ 05 AttributeSchema: testing get/set/add/del Readers """
        readers = ['SELF', 'ANON', 'AUTH',
                   'MNGR', 
                   'Divisional Administrators',
                   'System Administrators' ]
        self.failUnless( listsAreEquivalent( self.attrSchema.getReaders(),
                                             readers ) )

        readers = ['SELF', 'ANON', 'AUTH',]
        self.attrSchema.setReaders(readers)
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( listsAreEquivalent(self.attrSchema.getReaders(), 
                                            readers))

        readers = ['SELF', 'AUTH',]
        self.attrSchema.delReaders( 'ANON' )
        self.failUnless( listsAreEquivalent(self.attrSchema.getReaders(), 
                                            readers))

        readers = ['SELF', 'ANON', 'AUTH',]
        self.attrSchema.setReaders(readers)
        self.attrSchema.delReaders( ['ANON', 'AUTH'] )
        self.failUnless( listsAreEquivalent(self.attrSchema.getReaders(), 
                                            ['SELF',]))

    def test06(self):
        """ 06 AttributeSchema: testing get/set PointsTo with a valid value """
        self.attrSchema.setPointsTo('person')
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( self.attrSchema.getPointsTo() == 'person')

    def test07(self):
        """ 07 AttributeSchema: testing get/set PointsTo with a bad value """
        self.assertRaises(AssertionError,
                          self.attrSchema.setPointsTo,
                          'badValue')

    def test08(self):
        """ 08 AttributeSchema: testing get/set/add/del Writers """
        writers = ['SELF', 
                   'Divisional Administrators',
                   'System Administrators' ]
        self.failUnless( listsAreEquivalent(self.attrSchema.getWriters(), 
                                            writers))

        writers = ['SELF', 'ANON', 'AUTH',]
        self.attrSchema.setWriters(writers)
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( listsAreEquivalent(self.attrSchema.getWriters(), 
                                            writers))

        writers = ['SELF', 'AUTH',]
        self.attrSchema.delWriters( 'ANON' )
        self.failUnless( listsAreEquivalent(self.attrSchema.getWriters(), 
                                            writers))

        writers = ['SELF', 'ANON', 'AUTH',]
        self.attrSchema.setWriters(writers)
        self.attrSchema.delWriters( ['ANON', 'AUTH'] )
        self.failUnless( listsAreEquivalent(self.attrSchema.getWriters(), 
                                            ['SELF',]))

    def test09(self):
        """ 09 AttributeSchema: testing getRequired """
        self.failUnless( self.attrSchema.getRequired() )
        self.attrSchema = ManagedAttributeSchema('businessCategory',
                                       defaultUserSchema['businessCategory'])
        self.failIf( self.attrSchema.getRequired() )

    def test10(self):
        """ 10 AttributeSchema: testing get/set/add/del RequiredValues """
        self.failUnless( self.attrSchema.getRequiredValues() == [] )

        values = ['me', 'you']
        self.attrSchema.setRequiredValues(values)
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( listsAreEquivalent(self.attrSchema.getRequiredValues(),
                                            values ) )

        values = ['me', 'you', 'they' ]
        self.attrSchema.addRequiredValues( 'they' )
        self.failUnless( listsAreEquivalent(self.attrSchema.getRequiredValues(),
                                            values ) )
  
        values = ['me', 'you']
        self.attrSchema.delRequiredValues( 'they' )
        self.failUnless( listsAreEquivalent(self.attrSchema.getRequiredValues(),
                                            values ) )

        values = ['me', 'you', 'they', 'we' ]
        self.attrSchema.addRequiredValues( ['they', 'we'] )
        self.failUnless( listsAreEquivalent(self.attrSchema.getRequiredValues(),
                                            values ) )

        values = ['me', 'you']
        self.attrSchema.delRequiredValues( ['they', 'we'] )
        self.failUnless( listsAreEquivalent(self.attrSchema.getRequiredValues(),
                                            values ) )

        self.assertRaises(AssertionError,
                          self.attrSchema.delRequiredValues,
                          'badValue')

    def test11(self):
        """ 11 AttributeSchema: testing get/set syntax by name """
        syntaxName = "DirectoryString"
        self.attrSchema.setSyntaxByName(syntaxName)
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( syntaxName == self.attrSchema.getSyntaxName() )

    def test12(self)        :
        """ 12 AttributeSchems: testing get/set syntax by oid """
        oid = '1.3.6.1.4.1.1466.115.121.1.15'
        self.attrSchema.setSyntaxByOID(oid)
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( oid == self.attrSchema.getSyntaxOID() )

    def test13(self):
        """ 13 AttributeSchema: testing setSyntaxByName - getSyntaxOID """
        name = 'DirectoryString'
        oid = STLDAPSyntax.getOIDFromName(name)
        self.attrSchema.setSyntaxByName(name)
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( oid == self.attrSchema.getSyntaxOID() )

    def test14(self):
        """ 14 AttributeSchema: testing setSyntaxByOID - getSyntaxName """
        name = 'INTEGER'
        oid = STLDAPSyntax.getOIDFromName(name)
        self.attrSchema.setSyntaxByOID(oid)
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( name == self.attrSchema.getSyntaxName() )

    def test15(self):
        """ 15 AttributeSchema: testing setSyntaxByOID with a bad OID """
        self.assertRaises(AssertionError,
                          self.attrSchema.setSyntaxByOID,
                          'bad oid')

    def test16(self):
        """ 16 AttributeSchema: testing setSyntaxByName with a bad name """
        self.assertRaises(AssertionError,
                          self.attrSchema.setSyntaxByName,
                          'bad name')

    def test17(self):
        """ 17 AttributeSchema: testing get/set/add/del/has ValidValues """
        self.failUnless( self.attrSchema.getValidValues() == [] )

        values = ['me', 'you']
        self.attrSchema.setValidValues(values)
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( listsAreEquivalent(self.attrSchema.getValidValues(),
                                            values ) )

        values = ['me', 'you', 'they' ]
        self.attrSchema.addValidValues( 'they' )
        self.failUnless( listsAreEquivalent(self.attrSchema.getValidValues(),
                                            values ) )
  
        values = ['me', 'you']
        self.attrSchema.delValidValues( 'they' )
        self.failUnless( listsAreEquivalent(self.attrSchema.getValidValues(),
                                            values ) )

        values = ['me', 'you', 'they', 'we' ]
        self.attrSchema.addValidValues( ['they', 'we'] )
        self.failUnless( listsAreEquivalent(self.attrSchema.getValidValues(),
                                            values ) )

        values = ['me', 'you']
        self.attrSchema.delValidValues( ['they', 'we'] )
        self.failUnless( listsAreEquivalent(self.attrSchema.getValidValues(),
                                            values ) )

        self.assertRaises(AssertionError,
                          self.attrSchema.delValidValues,
                          'badValue')

        self.attrSchema.setValidValues( [] )
        self.failIf( self.attrSchema.hasValidValues() )

        self.attrSchema.setValidValues( [' ',] )
        self.failIf( self.attrSchema.hasValidValues() )

        self.attrSchema.setValidValues( ['a',] )
        self.failUnless( self.attrSchema.hasValidValues() )

    def test18(self):
        """ 18 AttributeSchema: testing get/set/add/del DisplayCategories """
        values = ['General', 'Personal',]
        self.failUnless( listsAreEquivalent(
                         self.attrSchema.getDisplayCategories(), values ) )

        values = ['me', 'you']
        self.attrSchema.setDisplayCategories(values)
        self.failUnless( self.attrSchema.MetadataChanged() )
        self.failUnless( listsAreEquivalent(
                         self.attrSchema.getDisplayCategories(), values ) )

        values = ['me', 'you', 'they' ]
        self.attrSchema.addDisplayCategories( 'they' )
        self.failUnless( listsAreEquivalent(
                         self.attrSchema.getDisplayCategories(), values ) )
  
        values = ['me', 'you']
        self.attrSchema.delDisplayCategories( 'they' )
        self.failUnless( listsAreEquivalent(
                         self.attrSchema.getDisplayCategories(), values ) )

        values = ['me', 'you', 'they', 'we' ]
        self.attrSchema.addDisplayCategories( ['they', 'we'] )
        self.failUnless( listsAreEquivalent(
                         self.attrSchema.getDisplayCategories(), values ) )

        values = ['me', 'you']
        self.attrSchema.delDisplayCategories( ['they', 'we'] )
        self.failUnless( listsAreEquivalent(
                         self.attrSchema.getDisplayCategories(), values ) )

        self.assertRaises(AssertionError,
                          self.attrSchema.delDisplayCategories,
                          'badValue')

    def test19(self):
        """ 19 AttributeSchema: testing the getMetadata function """
        self.attrSchema.setDisplayCategories(['cat1', 'cat2'])
        self.attrSchema.setFriendlyName('Common Name')
        self.attrSchema.setSyntaxByName('DirectoryString')
        self.attrSchema.setReaders(['me', 'you'])
        metadata = self.attrSchema.getMetadata()
        keys = ['syntax', 'required', 'pointsTo', 'friendlyName',
                'readers', 'validValues', 'requiredValues', 'managed', 
                'writers', 'displayCategories']
        self.failUnless( listsAreEquivalent(keys, metadata.keys()) )
        self.failUnless( listsAreEquivalent( ['cat1', 'cat2'],
                                             metadata['displayCategories'] ))
        self.failUnless( metadata['friendlyName'] == 'Common Name' )
        self.failUnless( metadata['syntax'] ==
                         STLDAPSyntax.getOIDFromName('DirectoryString'))
        self.failUnless( listsAreEquivalent(metadata['readers'], ['me', 'you']))

    def test20(self):
        """ 20 AttributeSchema: test getMetadata """
        metaOut = self.attrSchema.getMetadata()
        metaIn = defaultUserSchema['cn']
        self.failUnless(metaIn['friendlyName'] == metaOut['friendlyName'])
        self.failUnless(listsAreEquivalent(metaIn['readers'],
                                           metaOut['readers']))
        self.failUnless(listsAreEquivalent(metaIn['writers'],
                                           metaOut['writers']))
        self.failUnless(metaIn['required'] == metaOut['required'])
        self.failUnless(listsAreEquivalent(metaIn['displayCategories'],
                                           metaOut['displayCategories']))

    def test21(self):
        """ 21 AttributeSchema: test creating an attribute """
        metadata = {'friendlyName' : 'Common Name',
                    'required'     : TRUE }
        values = ['me', 'myself', 'I']
        testAttr = ManagedAttribute('cn', values=values, metadata=metadata)
        self.failUnless(testAttr.getAttributeName() == 'cn')
        self.failUnless(testAttr.getFriendlyName() == metadata['friendlyName'])
        self.failUnless(testAttr.getRequired() == metadata['required'])
        self.failUnless(listsAreEquivalent(testAttr.getValues(), values))
        testAttr = None

    def test22(self):
        """ 22 AttributeSchema: test of get/set Values """
        values = ['me', 'myself', 'I']
        testAttr = ManagedAttribute('cn')
        testAttr.setValues(values)
        self.failUnless(testAttr.ValueChanged())
        self.failUnless(listsAreEquivalent(testAttr.getValues(), values))
        testAttr = None

    def test23(self):
        """ 23 AttributeSchema: test of get/set Required """
        value = self.attrSchema.getRequired()
        self.attrSchema.setRequired( not value )
        self.failIf( value == self.attrSchema.getRequired())
        
    def test24(self):
        """ 24 AttributeSchema: test of displayValuesAsTextLines """
        testAttr = ManagedAttribute('cn')

        display = testAttr.displayValuesAsTextLines( [] )
        self.failUnless( display == "" )

        display = testAttr.displayValuesAsTextLines( ['a'] )
        self.failUnless( display == "a" )

        display = testAttr.displayValuesAsTextLines( ['a','b'] )
        self.failUnless( display == "a\nb" )

    def test25(self):
        """ 25 AttributeSchema: test of displayValuesAsHTMLLines """
        testAttr = ManagedAttribute('cn')

        display = testAttr.displayValuesAsHTMLLines( [] )
        self.failUnless( display == "" )

        display = testAttr.displayValuesAsHTMLLines( ['a'] )
        self.failUnless( display == "a" )

        display = testAttr.displayValuesAsHTMLLines( ['a','b'])
        self.failUnless( display == "a<br>\nb" )

    def test26(self):
        """ 26 AttributeSchema: test of displayValuesAsPassword """
        testAttr = ManagedAttribute('cn')

        display = testAttr.displayValuesAsPassword( FALSE )
        self.failUnless( display == "" )

        display = testAttr.displayValuesAsPassword( TRUE )
        self.failUnless( display == """
      <input type="password" name="cn1" size="35" value="Password"><br>
      <input type="password" name="cn2" size="35" value="Password">""" )

    def test27(self):
        """ 27 AttributeSchema: test of displayValuesAsTextarea """
        testAttr = ManagedAttribute('cn')

        display = testAttr.displayValuesAsTextarea( [] )
        self.failUnless( display == """<textarea name="cn:lines" rows="1" cols="35"></textarea>""" )

        display = testAttr.displayValuesAsTextarea( ['a','b'] )
        self.failUnless( display == """<textarea name="cn:lines" rows="3" cols="35">a
b</textarea>""" )

        display = testAttr.displayValuesAsTextarea( ['a','b','c','d'] )
        self.failUnless( display == """<textarea name="cn:lines" rows="3" cols="35">a
b
c
d</textarea>""" )

    def test28(self):
        """ 28 AttributeSchema: test of displayValuesAsOptionList """
        self.attrSchema = ManagedAttributeSchema('cn',defaultUserSchema['cn'])
        self.attrSchema.setValidValues( ['a','b','c'] )

        metadata=self.attrSchema.getMetadata()
        testAttr = ManagedAttribute('cn', metadata=metadata )

        display = testAttr.displayValuesAsOptionList( [] )
        self.failUnless( display == """
        <option>a</option>
        <option>b</option>
        <option>c</option>""" )

        display = testAttr.displayValuesAsOptionList( ['a'] )
        self.failUnless( display == """
        <option SELECTED>a</option>
        <option>b</option>
        <option>c</option>""" )

        display = testAttr.displayValuesAsOptionList( ['a'], 'CHECKED' )
        self.failUnless( display == """
        <option CHECKED>a</option>
        <option>b</option>
        <option>c</option>""" )

    def test29(self):
        """ 29 AttributeSchema: test of displayValuesAsMultipleSelection """
        self.attrSchema = ManagedAttributeSchema('cn',defaultUserSchema['cn'])
        self.attrSchema.setValidValues( ['a','b','c'] )

        metadata=self.attrSchema.getMetadata()
        testAttr = ManagedAttribute('cn', metadata=metadata )

        display = testAttr.displayValuesAsMultipleSelection( [] )
        self.failUnless( display == """
      <select name="cn:list" multiple size="3">
        <option>a</option>
        <option>b</option>
        <option>c</option>
      </select>""" )

        display = testAttr.displayValuesAsMultipleSelection( ['a'])
        self.failUnless( display == """
      <select name="cn:list" multiple size="3">
        <option SELECTED>a</option>
        <option>b</option>
        <option>c</option>
      </select>""" )

    def test30(self):
        """ 30 AttributeSchema: test of displayValuesAsLinks """

        metadata=self.attrSchema.getMetadata()
        testAttr = ManagedAttribute('cn', metadata=metadata )

        display = testAttr.displayValuesAsLinks( [], 'base/')
        self.failUnless( display == "" )

        display = testAttr.displayValuesAsLinks( ['a','b'], 'base/')
        self.failUnless( display == """<a href="base/a">a</a><br>
      <a href="base/b">b</a>""" )

    def test31(self):
        """ 31 AttributeSchema: test of attribute inheritance (OpenLDAP) """
        metadata = {'friendlyName' : 'Common Name',
                    'sup'          : 'name',
                    'required'     : TRUE }
        values = ['me', 'myself', 'I']
        testAttr = ManagedAttribute('cn', values=values, metadata=metadata)

        self.failUnless( testAttr.getSup() == 'name' )
        self.failUnless( self.attrSchema.getSup() == '' )

        metadata = {'syntax' : '1.3.6.1.4.1.1466.115.121.1.15{32768}'}
        supAttr = ManagedAttribute('name', values=[], metadata=metadata)
        testAttr.inheritsFrom( supAttr )
        self.failUnless( testAttr.getSyntaxOID() == metadata['syntax'] )

    def tearDown(self):
        self.attrSchema = None

if __name__ == '__main__':

    tests = unittest.makeSuite(testAttributeSchema, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
