"""

    Module: $RCSfile: TestAuthenticatedUser.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/04/18 14:54:38 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.10 $'[11:-2]

import sys
sys.path.insert(0,'..')

from TestDirectory import getTestDirectory
from Connection import *
from STLDAPUtilities import *
import unittest

class testAuthenticatedUser(unittest.TestCase):
    """ Tests the Authenticated User class in the Conection module """

    def setUp(self):
        self.directory = getTestDirectory()
        self.user = AuthenticatedUser('testadmin', 'testadmin',
                       self.directory)

    def test01(self):
        """01 testAuthenticatedUser: Logon test """
        self.failUnless(self.user.distinguishedName)

    def test02(self):
        """ 02 testAuthenticatedUser: Logon Failure Test """
        self.assertRaises(NO_SUCH_OBJECT, AuthenticatedUser, 'noname', 'wrongpassword',
                              self.directory)

    def test03(self):
        """ 03 testAuthenticatedUser: Specific Attribute Test """
        self.failUnless(self.user.attribute('cn'))

    def test04(self):
        """ 04 testAuthenticatedUser: Bad Specific Attribute Test """
        self.failIf(self.user.attribute('badAttribute'))

    def test05(self):
        """ 05 testAuthenticatedUser: Rename real user test """
        self.user.rename( 'uid=usera,ou=People,o=test', 'uid=bogusUser')
        self.user.rename( 'uid=bogusUser,ou=People,o=test', 'uid=usera' )

    def test06(self):
        """ 06 testAuthenticatedUser: Rename user with collision """
        self.assertRaises(RENAME_ERROR,
                          self.user.rename,
                          'uid=usera,ou=People,o=test',
                          'uid=userb' )

    def test07(self):
        """ 07 testAuthenticatedUser: testing getAllGroups """
        testGroups = ['Groups',
                      'Directory Administrators',
                      'Divisional Administrators',
                      'Empty Group',
                      'Manager',
                      'System Administrators',
                      'Test Group']
        groups = self.user.getAllGroups('ou=Groups,o=test')
        groups = [getNameFromDN(item) for item in groups]
        self.failUnless(listsAreEquivalent(groups, testGroups))

    def test08(self):
        """ 08 testAuthendicatedUser: testing _getZopeRoles """
        roles = ['ANON',
                 'AUTH',
                 'System Administrators',
                 'Divisional Administrators',]
        self.failUnless( listsAreEquivalent(roles, self.user.getRoles()))

    def tearDown(self):
        self.user.unbind()

if __name__ == '__main__':
    
    tests = unittest.makeSuite(testAuthenticatedUser, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
