"""

    Module: $RCSfile: TestLDIFAttributeParser.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/04/18 14:54:38 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.5 $'[11:-2]

import sys
sys.path.insert(0,'..')

import re
import unittest
from LDIFAttributeParser import LDIFAttributeParser

whitespacePat = re.compile(r'\s+')

class testLDIFAttributeParser(unittest.TestCase):

    def setUp(self):
        pass
    
    def tearDown(self):
        pass

    def test01(self):
        """ 01 testLDIFAttributeParser: testing nsDS5ReplicaCredentials """
        ldif = "( 2.16.840.1.113730.3.1.582 NAME 'nsDS5ReplicaCredentials' " \
             + "DESC 'iPlanet defined attribute type' " \
             + "SYNTAX 1.3.6.1.4.1.1466.115.121.1.5 SINGLE-VALUE " \
             + "X-ORIGIN 'iPlanet Directory Server' )"
        obj = LDIFAttributeParser(ldif)
        self.failUnless( ldif == obj.getLDIF())
        
    def test02(self):
        """ 02 testLDIFAttributeParser: testing javaClassName """
        ldif = "( 1.3.6.1.4.1.42.2.27.4.1.6 NAME 'javaClassName' " \
             + "DESC 'Fully qualified name of distinguished Java class or interface' " \
             + "EQUALITY caseExactMatch SYNTAX 1.3.6.1.4.1.1466.115.121.1.15 " \
             + "SINGLE-VALUE X-ORIGIN 'RFC 2713' )"
        self.failUnless( ldif == LDIFAttributeParser(ldif).getLDIF())
 
    def test03(self):
        """ 03 testLDIFAttributeParser: testing nsBCRelatedAutoload """
        ldif = "( 2.16.840.1.113730.3.1.430 NAME 'nsBCRelatedAutoload' " \
             + "DESC 'Netscape defined attribute type' " \
             + "SYNTAX 1.3.6.1.4.1.1466.115.121.1.15 " \
             + "X-ORIGIN 'Netscape Mission Control Desktop - Browser' )"
        self.failUnless( ldif == LDIFAttributeParser(ldif).getLDIF())

    def test04(self):
        """ 04 testLDIFAttributeParser: testing an attribute with aliases """
        ldif = "( 0.9.2342.19200300.100.1.3 NAME ( 'mail' 'rfc822mailbox' ) " \
             + "DESC 'Standard LDAP attribute type' " \
             + "SYNTAX 1.3.6.1.4.1.1466.115.121.1.15 X-ORIGIN 'RFC 1274' )"
        self.failUnless( ldif == LDIFAttributeParser(ldif).getLDIF())

    def test05(self):
        """ 05 testLDIFAttributeParser: testing OpenLDAP uniqueIdentifier """
        ldif = "( 0.9.2342.19200300.100.1.44 NAME 'uniqueIdentifier'\n" \
             + "\tEQUALITY caseIgnoreMatch\n" \
             + "\tSYNTAX 1.3.6.1.4.1.1466.115.121.1.15{256} )"
        ldifNoWS = re.sub( whitespacePat, ' ', ldif )
        self.failUnless( ldifNoWS == LDIFAttributeParser(ldif).getLDIF())

    def test06(self):
        """ 06 testLDIFAttributeParser: testing OpenLDAP associatedName """
        ldif = "( 0.9.2342.19200300.100.1.38 NAME 'associatedName'\n" \
             + "\tEQUALITY distinguishedNameMatch\n" \
             + "\tSYNTAX 1.3.6.1.4.1.1466.115.121.1.12 )"
        ldifNoWS = re.sub( whitespacePat, ' ', ldif )
        self.failUnless( ldifNoWS == LDIFAttributeParser(ldif).getLDIF())

    def test07(self):
        """ 07 STLDAPSyntax: testing OpenLDAP SUP syntax """
        ldif = """( 2.5.4.4 NAME ( 'sn' 'surname' ) SUP name )"""
        self.failUnless( ldif == LDIFAttributeParser(ldif).getLDIF())
        
if __name__ == '__main__':
    
    tests = unittest.makeSuite(testLDIFAttributeParser, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
