"""

    Module: $RCSfile: TestRequest.py,v $
    Author: $Author: stevray $
    Date:   $Date: 2002/02/13 18:24:08 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.2 $'[11:-2]

import sys
sys.path.insert(0, '..')
from RequestTest import request
from STLDAPUtilities import *
import unittest

class testRequest(unittest.TestCase):
    """
        Tests the request class in the RequestTest module.
        The request class emulates Zope's REQUEST object
        and is only used for testing python code that
        is integrated with Zope and needs access to a
        valid REQUEST object in order to be tested.
    """

    def setUp(self):
        self.REQUEST = request()

    def test01(self):
        """ 01 TestRequest:  testing the set/get methods """
        self.REQUEST.set('newKey', 'newValue')
        self.failUnless( 'newValue' == self.REQUEST.get('newKey') )

    def test02(self):
        """ 02 TestRequest:  testing get for form data """
        self.REQUEST.form['newKey'] = 'testValue'
        self.failUnless('testValue' == self.REQUEST.get('newKey') )

    def test03(self):
        """ 03 TestRequest:  testing has_key """
        self.REQUEST.set('key1', 'value1')
        self.REQUEST.form['key2'] = 'value2'
        self.REQUEST.cookies['key3'] = 'value3'
        self.failUnless( self.REQUEST.has_key('key1') )
        self.failUnless( self.REQUEST.has_key('key2') )
        self.failUnless( self.REQUEST.has_key('key3') )
        
    def test04(self):
        """ 04 TestRequest:  testing __setitem__/__getitem """
        self.REQUEST['key1'] = 'value1'
        self.failUnless( self.REQUEST['key1'] == 'value1' )
        
    def test05(self):
        """ 05 TestRequest:  testing keys() """
        keys = ['key1', 'key2', 'key3']
        self.REQUEST.set('key1', 'value1')
        self.REQUEST.form['key2'] = 'value2'
        self.REQUEST.cookies['key3'] = 'value3'
        self.failUnless( listsAreEquivalent(self.REQUEST.keys(), keys ) )  

    def test06(self):
        """ 06 TestRequest:  testing values() """
        values = ['value1', 'value2', 'value3']
        self.REQUEST.set('key1', 'value1')
        self.REQUEST.form['key2'] = 'value2'
        self.REQUEST.cookies['key3'] = 'value3'
        self.failUnless( listsAreEquivalent(self.REQUEST.values(), values ) )
        
    def test07(self):
        """ 07 TestRequest:  testing items() """
        items = [('key1', 'value1'), ('key2', 'value2'), ('key3', 'value3')]
        self.REQUEST.set('key1', 'value1')
        self.REQUEST.form['key2'] = 'value2'
        self.REQUEST.cookies['key3'] = 'value3'
        self.failUnless( listsAreEquivalent(self.REQUEST.items(), items ) )
        
    def test08(self):
        """ 08 TestRequest:  testing clear() """
        self.REQUEST.set('key1', 'value1')
        self.REQUEST.form['key2'] = 'value2'
        self.REQUEST.cookies['key3'] = 'value3'
        self.REQUEST.clear()
        self.failUnless( listsAreEquivalent(self.REQUEST.items(), [] ) )
        
    def test09(self):
        """ 09 TestRequest:  testing copy() """
        items = [('key1', 'value1'), ('key2', 'value2'), ('key3', 'value3')]
        self.REQUEST.set('key1', 'value1')
        self.REQUEST.form['key2'] = 'value2'
        self.REQUEST.cookies['key3'] = 'value3'
        tmp = self.REQUEST.copy()
        self.failUnless( listsAreEquivalent(tmp.items(), items ) )
        
    def test10(self):
        """ 10 TestRequest:  testing __contains__() """
        self.REQUEST.set('key1', 'value1')
        self.REQUEST.form['key2'] = 'value2'
        self.REQUEST.cookies['key3'] = 'value3'
        self.failUnless( 'key2' in self.REQUEST )
        
    def test11(self):
        """ 11 TestRequest:  testing __len__() """
        self.REQUEST.set('key1', 'value1')
        self.REQUEST.form['key2'] = 'value2'
        self.REQUEST.cookies['key3'] = 'value3'
        self.failUnless( len(self.REQUEST) == 3 )
        
    def test12(self):
        """ 12 TestRequest:  testing __delitem__() """
        self.REQUEST.set('key1', 'value1')
        self.REQUEST.form['key2'] = 'value2'
        self.REQUEST.cookies['key3'] = 'value3'
        del self.REQUEST['key2']
        self.failIf( 'key2' in self.REQUEST )
        
    def tearDown(self):
        self.REQUEST = None

if __name__ == '__main__':

    tests = unittest.makeSuite(testRequest, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)