"""

    Module: $RCSfile: TestUtilities.py,v $
    Author: $Author: stevray $
    Date:   $Date: 2002/02/13 18:24:08 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.6 $'[11:-2]

import sys
sys.path.insert(0,'..')

import unittest
from STLDAPUtilities import *
from LDAPExceptions import INVALID_DN_SYNTAX

class testUtilities(unittest.TestCase):

    def setUp(self):
        self.list1 = ['a', 'b']
        self.list2 = ['b', 'a']

    def test01(self):
        """ 01 STLDAPUtilities: testing listsAreEquivalent with equal lists """
        
        self.failUnless( listsAreEquivalent(self.list1,self.list2) )

    def test02(self):
        """ 02 STLDAPUtilities: testing listsAreEquivalent with empty lists """
        self.failUnless( listsAreEquivalent( [], [] ) )

    def test03(self):
        """ 03 STLDAPUtilities: testing listsAreEquivalent with duplicate entries """
        self.failIf( listsAreEquivalent( ['a', 'b',], ['a', 'b', 'b',] ) )

    def test04(self):
        """ 04 STLDAPUtilities: testing listsAreEquivalent type safety """
        self.assertRaises(AssertionError, listsAreEquivalent, ['a'], '')
        self.assertRaises(AssertionError, listsAreEquivalent, '', ['a'])

    def test05(self):
        """ 05 STLDAPUtilities: testing listCopy function """
        tmp =listCopy(self.list1)
        self.failUnless( listsAreEquivalent(tmp, self.list1) )
        tmp = tmp + ['c']
        self.failIf( listsAreEquivalent(tmp, self.list1) )

    def test06(self):
        """ 06 STLDAPUtilities: testing listCopy type safety """
        self.assertRaises(AssertionError, listCopy, 'a')

    def test07(self):
        """ 07 STLDAPUtilities: testing list1ContainsList2 """
        list1 = ['a', 'b', 'c', 'd']
        list2 = ['b', 'c']
        self.failUnless( list1ContainsList2( list1, list2 ) )
        list2 = ['d', 'e']
        self.failIf( list1ContainsList2( list1, list2 ) )

    def test08(self):
        """ 08 STLDAPUtilities: testing getNameFromDN """
        dn = 'cn = West is Best,ou=ST ScI, o=stsci.edu'
        self.failUnless( 'West is Best' == getNameFromDN(dn) )

    def test09(self):
        """ 09 STLDAPUtilities: testing getNameFromDN with bad dn """
        badDN = 'West is Best,ou=ST ScI, o= stsci.edu'
        self.assertRaises(INVALID_DN_SYNTAX, getNameFromDN, badDN)

    def test10(self):
        """ 10 STLDAPUtilities: testing DNNormalize """
        dn = 'cn = West is Best,ou=ST ScI, o=stsci.edu'
        normalized = 'cn=west is best,ou=st sci,o=stsci.edu'

        self.failUnless( DNNormalize( dn ) == normalized )

        dn = 'cn = West is Best ,ou=ST ScI, o=stsci.edu'
        self.failUnless( DNNormalize( dn ) == normalized )

    def test11(self):
        """ 11 STLDAPUtilities: testing DNsAreEquivalent """
        dn1 = 'cn = West is Best,ou=ST ScI, o=stsci.edu'
        dn2 = 'cn=West is Best ,ou=ST SCI , o =stsci.edu'
        self.failUnless( DNsAreEquivalent( dn1, dn2 ) )

        dn2 = 'cn=West is Best ,ou=ST  SCI , o =stsci.edu'
        self.failIf( DNsAreEquivalent( dn1, dn2 ) )


if __name__ == '__main__':
    
    tests = unittest.makeSuite(testUtilities, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
