"""

    Module: $RCSfile: TestZSchema.py,v $
    Author: $Author: payne $
    Date:   $Date: 2002/04/18 14:54:38 $
    
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
__version__ = '$Revision: 1.22 $'[11:-2]

import sys
sys.path.insert(0,'..')

import unittest

from TestDirectory import getTestDirectory
from Connection import AuthenticatedUser
from LDAPExceptions import REQUIRED_ATTRIBUTE, NO_SUCH_ATTRIBUTE
from STLDAPUtilities import listsAreEquivalent
from ManagedRoles import ManagedRoles
from ManagedSchema import ManagedSchema
from SchemaDefaults import *
from TestDirectory import getTestDirectory
from ZSchema import ZSchema

class testZSchema(unittest.TestCase):

    def setUp(self):
        directory = getTestDirectory()
        dict = { 'objectCategory' : 'person' }
        self.user = AuthenticatedUser('testadmin', 'testadmin',
                       directory)
        self.ZSchema = ZSchema( self.user, dict )


    def test01(self):
        """ 01 TestZSchema: testing checkObjectClassCompleteness """
        self.failUnless( self.ZSchema.checkObjectClassCompleteness(
                         ['top', 'person', 'organizationalPerson'] ) )
        self.failIf( self.ZSchema.checkObjectClassCompleteness(
                     ['top', 'person', 'inetOrgPerson'] ) )

    def test02(self):
        """ 02 TestZSchema: testing getUnmanagedObjectClasses """
        unmanaged = [
            'account', 'alias', 'applicationentity',
            'applicationprocess', 'automount', 'bootabledevice',
            'cacheobject', 'calentry', 'certificationauthority',
            'certificationauthority-v2', 'changelogentry',
            'cirreplicasource', 'cosclassicdefinition',
            'cosdefinition', 'cosindirectdefinition',
            'cospointerdefinition', 'cossuperdefinition',
            'costemplate', 'country', 'crldistributionpoint',
            'dcobject', 'device', 'directoryserverfeature', 'dmd',
            'dnsdomain', 'document', 'documentseries', 'domain',
            'domainrelatedobject', 'dsa', 'extensibleobject',
            'friendlycountry', 'glue', 'groupofcertificates',
            'groupofmailenhanceduniquenames', 'groupofnames',
            'groupofuniquenames', 'groupofurls', 'ieee802device',
            'inetadmin', 'inetdomain', 'inetorgperson',
            'inetsubscriber', 'inetuser', 'iphost',
            'iplanetlinkedorganization', 'iplanetorgservice',
            'iplanetpreferences', 'iplanetreversiblepasswordobject',
            'iplanetservice', 'iplanetservicecomponent',
            'iplanetserviceplugin', 'ipnetwork', 'ipprotocol',
            'ipservice', 'javacontainer', 'javamarshalledobject',
            'javanamingreference', 'javaobject',
            'javaserializedobject', 'labeleduriobject',
            'ldapreplica', 'ldapserver', 'ldapsubentry',
            'locality', 'mailgroup', 'mailgroupmanagement',
            'mailgroupmanagement-globalconfig', 'mailgroupmember',
            'mailrecipient', 'netscapecalendarserver',
            'netscapecertificateserver', 'netscapecompassserver',
            'netscapedirectoryserver', 'netscapemachinedata',
            'netscapemailserver', 'netscapemediaserver',
            'netscapemessagingserver', 'netscapenewsserver',
            'netscapepreferencemap', 'netscapeproxyserver',
            'netscapeserver', 'netscapewebserver', 'netshareaccount',
            'netshareprojectmanagementglobalconfig', 'newpilotperson',
            'nginfo', 'nismap', 'nisnetgroup', 'nisobject',
            'nsadminconfig', 'nsadminconsoleuser', 'nsadmindomain',
            'nsadminglobalparameters', 'nsadmingroup', 'nsadminobject',
            'nsadminresourceeditorextension', 'nsadminserver',
            'nsapplication', 'nsbackendinstance', 'nsbrowserclient',
            'nsbrowsersecurity', 'nscaladmin', 'nscalresource',
            'nscaluser', 'nscertificateserver', 'nschangelog4config',
            'nscomplexroledefinition', 'nsconfig', 'nsconsumer4config',
            'nscontainer', 'nscustomview', 'nsdefaultobjectclasses',
            'nsdirectoryinfo', 'nsdirectoryserver', 'nsds5replica',
            'nsds5replicationagreement', 'nsencryptionconfig',
            'nsencryptionmodule', 'nsfilteredroledefinition',
            'nsglobalparameters', 'nshost', 'nsindex',
            'nslegacyadmingroup', 'nslegacyadminserver',
            'nslegacyapplication', 'nslegacyserver', 'nslicenseuser',
            'nsliprofile', 'nsliprofileelement', 'nsliptr',
            'nsliserver', 'nsmailclient', 'nsmailsecurity',
            'nsmanageddept', 'nsmanageddeptadmingroup',
            'nsmanageddomain', 'nsmanagedfamilygroup',
            'nsmanagedisp', 'nsmanagedmaillist', 'nsmanagedorgunit',
            'nsmanagedpeoplecontainer', 'nsmanagedperson',
            'nsmanagedroledefinition', 'nsmappingtree',
            'nsmcdconfig', 'nsmessagingserveruser', 'nsmsgcfgalarm',
            'nsmsgcfgalarmcontainer', 'nsmsgcfgalias',
            'nsmsgcfgcontainer', 'nsmsgcfgexpirerule',
            'nsmsgcfggen', 'nsmsgcfghttp', 'nsmsgcfgimap',
            'nsmsgcfglog', 'nsmsgcfgmta', 'nsmsgcfgmtaaccept',
            'nsmsgcfgmtaautoreplyhandler', 'nsmsgcfgmtaerrorhandler',
            'nsmsgcfgmtalog', 'nsmsgcfgmtamboxdeliver',
            'nsmsgcfgmtaprogdeliver', 'nsmsgcfgmtarouter',
            'nsmsgcfgmtasmtpdeliver', 'nsmsgcfgmtaunixdeliver',
            'nsmsgcfgnntp', 'nsmsgcfgpartition', 'nsmsgcfgpop',
            'nsmsgcfgpublic', 'nsmsgcfgreport', 'nsmsgcfgscript',
            'nsmsgcfgservice', 'nsmsgcfgsnmp', 'nsmsgcfgstore',
            'nsmsgcfguser', 'nsnestedroledefinition', 'nsresourceref',
            'nsroledefinition', 'nssimpleroledefinition',
            'nsslapdconfig', 'nsslapdplugin', 'nssnmp',
            'nstask', 'nstaskgroup', 'nstombstone',
            'nstopologycustomview', 'nstopologyplugin',
            'nsuniquenessdomain', 'nsvalueitem', 'nswcaluser',
            'ntgroup', 'ntuser', 'oncrpc', 'organization',
            'organizationalperson', 'organizationalrole',
            'organizationalunit', 'passwordobject',
            'passwordpolicy', 'personalinterestprofile',
            'pilotobject', 'pilotorganization', 'pipuser',
            'pipuserinfo', 'posixaccount', 'posixgroup',
            'referral', 'residentialperson', 'rfc822localpart',
            'room', 'shadowaccount', 'simplesecurityobject',
            'strongauthenticationuser', 'stsciorganizationalunit',
            'stsciperson', 'subschema', 'usersecurityinformation',
            'vlvindex', 'vlvsearch', ]
        unmanagedOpenLDAP = [
            'account', 'alias', 'applicationentity', 'applicationprocess',
            'certificationauthority', 'certificationauthority-v2',
            'country', 'crldistributionpoint', 'dcobject', 'device',
            'dmd', 'dnsdomain', 'document', 'documentseries', 'domain',
            'domainrelatedobject', 'dsa', 'dynamicobject', 'extensibleobject',
            'friendlycountry', 'groupofnames', 'groupofuniquenames', 
            'inetorgperson', 'labeleduriobject', 'ldapsubentry',
            'locality', 'openldaprootdse', 'organization',
            'organizationalperson', 'organizationalrole', 'organizationalunit',
            'pilotdsa', 'pilotorganization', 'pilotperson',
            'qualitylabelleddata', 'referral', 'residentialperson',
            'rfc822localpart', 'room', 'simplesecurityobject', 
            'strongauthenticationuser', 'subschema', 'uidobject',
            'usersecurityinformation', ]

        found = self.ZSchema.getUnmanagedObjectClasses()
        if 'iplanetservice' in found:
            self.failUnless (listsAreEquivalent(unmanaged, found))
        else:
            self.failUnless (listsAreEquivalent(unmanagedOpenLDAP, found))

    def test03(self):
        """ 03 TestZSchema: getManagedRoles """
        managedRoles = self.ZSchema.getManagedRoles()
        roles = self.user.getLDAPDirectory().getManagedRoles()
        self.failUnless( managedRoles == roles )

    def test04(self):
        """ 04 TestZSchema: getManagedSchema """
        schema = self.ZSchema.getManagedSchema()
        self.failUnless( schema == self.schema )

    def test04(self):
        """ 04 TestZSchema: update from directory """
        self.ZSchema.updateSchemaFromLDAP( self.user )

        allowed = [
            'audio', 'businessCategory', 'carLicense',
            'cn', 'departmentNumber', 'description',
            'destinationIndicator', 'displayName', 'employeeNumber',
            'employeeType', 'facsimileTelephoneNumber',
            'givenName', 'homePhone', 'homePostalAddress',
            'initials', 'internationaliSDNNumber', 'jpegPhoto',
            'l', 'labeledUri', 'mail', 'mailAccessDomain',
            'mailAlternateAddress', 'mailAutoReplyMode',
            'mailAutoReplyText', 'mailDeliveryOption',
            'mailForwardingAddress', 'mailHost',
            'mailMessageStore', 'mailProgramDeliveryInfo',
            'mailQuota', 'mailRoutingAddress', 'manager', 'mobile',
            'multiLineDescription', 'o', 'objectClass', 'ou',
            'pager', 'photo', 'physicalDeliveryOfficeName',
            'postOfficeBox', 'postalAddress', 'postalCode',
            'preferredDeliveryMethod', 'preferredLanguage',
            'registeredAddress', 'roomNumber', 'secretary',
            'seeAlso', 'sn', 'st', 'street', 'telephoneNumber',
            'teletexTerminalIdentifier', 'telexNumber',
            'title', 'uid', 'userCertificate', 'userPKCS12',
            'userPassword', 'userSMIMECertificate', 'x121Address',
            'x500UniqueIdentifier', ]

        allowedOpenLDAP = [
            'audio', 'businessCategory', 'carLicense',
            'cn', 'departmentNumber', 'description',
            'destinationIndicator', 'displayName', 'employeeNumber',
            'employeeType', 'facsimileTelephoneNumber',
            'givenName', 'homePhone', 'homePostalAddress',
            'initials', 'internationaliSDNNumber', 'jpegPhoto',
            'l', 'labeledURI', 'mail', 'manager', 'mobile',
            'o', 'objectClass', 'ou', 'pager', 'photo',
            'physicalDeliveryOfficeName', 'postOfficeBox',
            'postalAddress', 'postalCode', 'preferredDeliveryMethod',
            'preferredLanguage', 'registeredAddress', 'roomNumber',
            'secretary', 'seeAlso', 'sn', 'st', 'street',
            'telephoneNumber', 'teletexTerminalIdentifier',
            'telexNumber', 'title', 'uid', 'userCertificate',
            'userPKCS12', 'userPassword', 'userSMIMECertificate',
            'x121Address', 'x500uniqueIdentifier', ]

        schema = self.user.getLDAPDirectory().getPersonSchema()
        found = schema.getAllowedAttributeNames()
        if 'mailAccessDomain' in found:
            for item in allowed:
                self.failUnless( item in found )
                found.remove( item )
        else:
            for item in allowedOpenLDAP:
                self.failUnless( item in found )
                found.remove( item )
        self.failUnless( found == [])

        found = schema.getAttributeCollection()
        self.failUnless( listsAreEquivalent( 
                         schema.getAllowedAttributeNames(), found.keys()))

        attrSchema = schema.getAttributeSchema( 'cn' )
        self.failUnless( attrSchema.getRequired() )

        self.assertRaises(AssertionError,
                          schema.getAttributeSchema,
                          'badValue')

        available = [
            'audio', 'destinationIndicator',
            'employeeNumber', 'internationaliSDNNumber',
            'jpegPhoto', 'labeledUri', 'mailAccessDomain',
            'mailAutoReplyMode', 'mailAutoReplyText',
            'mailDeliveryOption', 'mailForwardingAddress',
            'mailMessageStore', 'mailProgramDeliveryInfo',
            'mailQuota', 'multiLineDescription', 'photo',
            'preferredDeliveryMethod', 'preferredLanguage',
            'registeredAddress', 'teletexTerminalIdentifier', 
            'telexNumber', 'userCertificate', 'userPKCS12', 
            'userSMIMECertificate', 'x121Address', 
            'x500UniqueIdentifier', ]

        availableOpenLDAP = [
            'audio', 'destinationIndicator', 'employeeNumber',
            'internationaliSDNNumber', 'jpegPhoto', 'labeledURI',
            'photo', 'preferredDeliveryMethod', 'preferredLanguage',
            'registeredAddress', 'teletexTerminalIdentifier',
            'telexNumber', 'userCertificate', 'userPKCS12',
            'userSMIMECertificate', 'x121Address',
            'x500uniqueIdentifier', ]

        found = schema.getAvailableAttributeNames()
        if 'mailAccessDomain' in found:
            for item in available:
                self.failUnless( item in found )
                found.remove( item )
        else:
            for item in availableOpenLDAP:
                self.failUnless( item in found )
                found.remove( item )
        self.failUnless( found == [] )

    def test05(self):
        """ 05 TestZSchema: testing that an LDAPUpdate forces the naming attribute to be required """
        schema = self.ZSchema.getManagedSchema()
        schema.setNamingAttribute('postalAddress')
        self.ZSchema.updateSchemaFromLDAP(self.user)
        self.failUnless( schema.getAttributeSchema( 'postalAddress' ).getRequired() )
 
    def test06(self):
        """ 06 TestZSchema: testing manageAttributeSchema """
        
        schema = self.ZSchema.getManagedSchema()
        managed = schema.getManagedAttributeNames()
        managed.append( 'audio' )
        schema.manageAttributeSchema( 'audio' )
        self.failUnless( listsAreEquivalent( managed,
                         schema.getManagedAttributeNames() ))

        self.assertRaises(AssertionError,
                          schema.manageAttributeSchema,
                          'badValue')

    def test07(self):
        """ 07 TestZSchema: testing _addAvailableObjectClasses """
        
        dict = {
            'managed'        : '1',
            'required'       : '1',
            'updateSchema'   : '1',
            'newClasses'     : ['residentialPerson',],
            'objectCategory' : 'person',
            'schemaFormID'   : 'schemaAttributes',
            'ldapName'       : 'objectClass',
            }
        schema = self.user.getLDAPDirectory().getPersonSchema()
        self.ZSchema._addAvailableObjectClasses( dict )
        objectClassSchema = schema.getAttributeSchema( 'objectClass' )
        valid = objectClassSchema.getValidValues()
        self.failUnless( listsAreEquivalent( valid,
                         ['top','person','organizationalPerson',
                          'inetOrgPerson', 'mailRecipient',
                          'residentialPerson']))

    def tearDown(self):
        self.user.unbind()

if __name__ == '__main__':

    tests = unittest.makeSuite(testZSchema, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
