__version__ = '$Id: '

"""
Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""
########################################################################
#
#                       Zope Unit Testing Fixture
#
#                           Don Stevens-Rayburn
#
#   This module provides a framework for performing unit tests of
#   Zope objects.
#
#   In order to use this framework, you must add the Zope /lib/python
#   directory to your PYTHONPATH environment variable.
#
#   In addition, Zope must not be running on the same machine that you
#   are using to test it.
#
#   This framework uses the unittest testing framework that is part of
#   the Python Standard Library for Python 2+ and is available as an
#   add-on for earlier versions of Python.  See Section 5.2 of the
#   Python Library Reference, Release 2.1 for more information on
#   using the unittest framework.
#
########################################################################

import Zope, ZPublisher
import unittest
from StringIO import StringIO
import sys

class TestSTLDAPManager(unittest.TestCase):

    def setUp(self):
        self.result = StringIO()
        self.hold = sys.stdout
        sys.stdout = self.result

    def tearDown(self):
        sys.stdout = self.hold
        # print self.result.getvalue()

    def test01(self):
        """ 01 Testing the default View method of the STLDAPManager   """

        url = '/STLDAP2/LDAP01'

        #   Now, we need to invoke Zope to publish the url.
        #   To do that, we make a call of the form:
        #
        #       ZPublisher.Zope(url,
        #                       [u='username:password',]
        #                       [e={'some': 'environment',
        #                           'variable' : 'settings' }] )

        ZPublisher.Zope(url,
                        u='stevray:junkstuff')        # Change this call as neccessary.

        #   Retrieve the result into a string variable.
        
        html = self.result.getvalue()

        #   The HTML generated by the Zope request is now in
        #   the variable html.  You now need to write code to parse
        #   that result.  Generally, all you need to do is to check
        #   to see if the result contains some particular string
        #   that you are looking for.  So, the code may be something
        #   like:

        self.failUnless(html.find('404 Not Found') == -1)

        #   This test succeeds if the string 'www.zope.org' was
        #   included in the HTML returned by Zope and reports a
        #   failure if it was not.

    def test02(self):
        """ 02 Testing AllowedAttributeNames """
        url = '/STLDAP2/LDAP01/allowedAttributeNames?objectCategory=group'
        ZPublisher.Zope(url)
        html = self.result.getvalue()
        self.failIf( html.find("'uniqueMember',") == -1)
                
        #   You may add as many different tests as you like.  The
        #   only restriction is that each test method name must
        #   begin with the string 'test'.

if __name__ == '__main__':
    
    tests = unittest.makeSuite(TestSTLDAPManager, 'test')
    unittest.TextTestRunner(verbosity=2).run(tests)
      
        
